# init.py
import os

def init() -> None:
    global FOLDER
    FOLDER = ""
    
    global SRC_DIR
    SRC_DIR = ""
    
    global APP_NAME
    APP_NAME = ""
    
    global APP_DIR
    APP_DIR = ""
    
    global GIT_REPO
    GIT_REPO = ""
    
    global CUSTOMER
    CUSTOMER = ""
    
    global ROOT_DIR
    ROOT_DIR = ""
    



# custom_method.py

import os
import base64

from src import settings


def creates_custom_methods(methods):
    for method in methods:
        event = CustomMethod(method, settings.APP_DIR + "/custom_methods/")
        event.to_file()

class CustomMethod(object):
    """
    Define a custom method
    """
    get = ""
    post = ""
    put = ""
    patch = ""

    def __init__(self, method, dir):
        self.id = method["id"]
        self.name = method.Name.cdata
        if hasattr(method, "GetB64"):
            self.get = method.GetB64.cdata
        if hasattr(method, "PostB64"):
            self.post = method.PostB64.cdata
        if hasattr(method, "PutB64"):
            self.put = method.PutB64.cdata
        if hasattr(method, "PatchB64"):
            self.patch = method.PatchB64.cdata
        self.endpoint = method.Endpoint.cdata
        self.method_dir = "{}/{}".format(dir, self.endpoint)

    def to_file(self):
        # properties file
        if not os.path.exists(self.method_dir):
            os.makedirs(self.method_dir)
        file = open("{}/PROPERTIES.md".format(self.method_dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- endpoint : <server_prod_url>/api2/custom/{}\n".format(self.endpoint))

        if self.get:
            file = open("{}/get.js".format(self.method_dir), 'w+', encoding='utf-8')
            with file:
                file_content = base64.b64decode(self.get).decode("UTF-8", "ignore")
                file.write(file_content)

        if self.post:
            file = open("{}/post.js".format(self.method_dir), 'w+', encoding='utf-8')
            with file:
                file_content = base64.b64decode(self.post).decode("UTF-8", "ignore")
                file.write(file_content)

        if self.put:
            file = open("{}/put.js".format(self.method_dir), 'w+', encoding='utf-8')
            with file:
                file_content = base64.b64decode(self.put).decode("UTF-8", "ignore")
                file.write(file_content)

        if self.patch:
            file = open("{}/patch.js".format(self.method_dir), 'w+', encoding='utf-8')
            with file:
                file_content = base64.b64decode(self.patch).decode("UTF-8", "ignore")
                file.write(file_content)




# role_page.py

from src import settings

def creates_role_pages(roles):
    t = open("{}/ROLE_PAGES.md".format(settings.APP_DIR), "w+", encoding="utf-8")
    with t:
        t.write("\n## Role page:\n")
        
        t.write("|pageDefId|pageId|roleId|\n|---|---|---|\n")
        for x in roles:
            #TODO: find pageDef name and Role name
            t.write("|{}|{}|{}|\n".format(x["pageDefId"], x["pageId"], x["roleId"]))


# permission.py

from src import settings

def creates_permissions(permissions):
    t = open("{}/PERMISSIONS.md".format(settings.APP_DIR), "w+", encoding="utf-8")
    with t:
        t.write("\n## Permissions :\n")
        
        t.write("|resId|prId|actionsSet|\n|---|---|---|\n")
        for x in permissions:
                t.write("|{}|{}|{}|\n".format(x["resId"], x["prId"], x["actionsSet"]))


import src.settings as settings, os, json
from src.utils import my_json

def creates_dependents(dependents):
    # Dependants objects
    readme = open("{}/DEPENDENTS.md".format(settings.APP_DIR), "w+", encoding="utf-8")
    with readme:
        readme.write("\n## Objets dépendants:\n")
        json_path = os.path.join(settings.SRC_DIR, "utils", "objects_dict.json")
        objects_dict = json.load(open(json_path))
        readme.write(
            "|Id d'origine|Id|Nom|Nom d'intégration|\n" "|---|---|---|---|\n"
        )

        # write in json file
        jsontest = json.dumps(objects_dict)
        with open(json_path, "w", encoding="utf-8") as f:
            f.write(jsontest)

        del jsontest

        for dependent in dependents:
            obj = my_json.find_in_json(objects_dict, dependent["id"])
            readme.write(
                "|{}|{}|{}|{}|\n".format(
                    dependent["origId"],
                    dependent["id"],
                    dependent.SingularName.cdata,
                    dependent["objDefName"],
                )
            )



# status.py
import os
from src.models._base_def import AField
from src.models.translation import creates_translations

from src.utils import split_tool


def creates_statuses(statuses, obj_dir) -> None:
    path = "{}/statuses".format(obj_dir)
    if not os.path.exists(path.encode("utf-8")):
        os.makedirs(path.encode("utf-8"))
    for status in statuses:
        Status(status, path).to_file()


class Status(AField):
    """ """

    translations = ""

    def __init__(self, data, path):
        super().__init__(data, path)

        self.order = data["orderNo"]
        self.obj_type_id = data["objTypeId"]

    def to_file(self):
        super().to_file()

        file = open("{}/STATUS.md".format(self.dir), "w+", encoding="utf-8")
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- orderNo : {}\n".format(self.order))
            file.write("- objDefId : {}\n".format(self.obj_def_id))
            file.write("- objTypeId : {}\n".format(self.obj_type_id))



# relationship.py

def creates_relationships(relations, obj_dir) -> None:
        # fichier des relations
        file = open("{}/RELATIONSHIP.md".format(obj_dir), 'w+', encoding='utf-8')
        with file:
            file.write("## Relations\n")
            sens_relation = "1 -- 1"
            for relation in relations:
                # 1 -- 1
                if relation["isMultiple"] == "false" and relation["isMultiple2"] == "false":
                    sens_relation = "1 -- 1"

                # 1 -- N
                if relation["isMultiple"] == "false" and relation["isMultiple2"] == "true":
                    sens_relation = "1 -- N"

                # N -- 1
                if relation["isMultiple"] == "true" and relation["isMultiple2"] == "false":
                    sens_relation = "N -- 1"

                # N -- N
                if relation["isMultiple"] == "true" and relation["isMultiple2"] == "true":
                    sens_relation = "1 -- 1"

                file.write("- {} : \n\t * [x] {}({}) `{}` {}({})\n".format(
                    relation["relName"],
                    relation["singularName1"],
                    relation["objDef1"],
                    sens_relation,
                    relation["singularName2"],
                    relation["objDef2"]
                ))


# usage.py

def usage():
    print("usage :")
    print("src -f <input_xml_file> -n <app_name> [-c <customer_id>]")
    print("\t-o, --out-folder=\t\toutput folder")
    print(
        "\t-n, --name=\t\tapplication name, permit application merge in case of name change"
        " (ex core_app"
    )
    print("\t-f, --file=\t\t:XML file to convert")
    print("\t-g, --git-repository-ssh=\t\t: ssh url of the repository")
    print("\t-c, --customer-id=\t\tOPTIONEL : customer id, to create a new branch")



import json
import os, re
from src.utils import my_json, split_tool
from src import settings

def creates_seeds(seeds):
    for seed in seeds:
        new_seed = SeedRecord(seed)
        new_seed.to_file()


class SeedRecord(object):
    """

    """

    def __init__(self, data_object):
        self.id = data_object["id"]
        self.obj_def_id = data_object["objDefId"]
        self.fields = data_object.Field
        self.obj_name = ""

        # obj_dict
        json_dict = os.path.join(settings.SRC_DIR, "utils", "objects_dict.json")
        file_dict = json.load(open(json_dict))
        obj = my_json.find_id_json(file_dict, self.obj_def_id)
        try:
            if obj:
                self.obj_name = split_tool.split_name(obj["name"])
        except Exception as err:
            print(err)

        self.seed_dir = self.portal_dir = "{}/seeds".format(settings.APP_DIR)

        if not os.path.exists(self.portal_dir.encode('utf-8')):
            os.makedirs(self.portal_dir.encode('utf-8'))

    def to_file(self):
        if self.obj_name:
            file = open("{}/{}.md".format(self.portal_dir, self.obj_name), 'a+', encoding='utf-8')
            with file:
                file.write("\n* {} : \n".format(self.id))
                for field in self.fields:
                    file.write("\t* {} : {}\n".format(field["name"], field.cdata))



from unittest import TestCase


class TestBaseField(TestCase):
    def test_create_dir(self):
        self.fail()

    def test_to_file(self):
        self.fail()

    def test_set_properties(self):
        self.fail()

    def test_create_validation_script(self):
        self.fail()



# template.py
import os
import base64
from src.models._base_def import AField
from src.utils import split_tool

def creates_templates(templates, obj_dir) -> None:
    path = "{}/templates".format(obj_dir)
    if not os.path.exists(path.encode('utf-8')):
        os.makedirs(path.encode('utf-8'))
    for template_data in templates:
        template = Template(template_data, path)
        template.to_file()

class Template(AField):
    """
    Define a template
    """
    raw_data = ""
    body = ""
    props = ""

    def __init__(self, data, path):
        super().__init__(data, path)
        self.type = data["type"]
        self.name = data["name"]
        self.content_type = data["contentType"]
        self.obj_type_id = data["objTypeId"]
        if self.type == "mail":
            self.subject = data.Subject.cdata
            self.body = data.Body.cdata
        if self.type == "doc":
            self.raw_data = data.RawData.cdata
            

    def to_file(self):
        super().to_file()
        # properties file

        file = open("{}/PROPERTIES.md".format(self.dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDef : {}\n".format(self.obj_def_id))
            file.write("- type : {}\n".format(self.type))
            file.write("- contentType : {}\n".format(self.content_type))
            file.write("- objTypeId : {}\n".format(self.obj_type_id))

            if self.properties:
            # Properties
                file.write("\n\n## Properties\n")
                for el in self.properties["children"]:
                    if((el.__dict__["_name"] != "Body" or el.__dict__["_name"] != "RawData") ) and (el.__dict__["_name"] != "Props"):
                        file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))


        if self.body:
            file = open("{}/body.html".format(self.dir), 'w+', encoding='utf-8')
            with file:
                file_content = base64.b64decode(self.body).decode("UTF-8", "ignore")
                file.write(file_content)
        
        if self.raw_data:
            name = "doc"
            if "text/plain" in self.content_type:
                name = name + ".txt"
            elif "application/vnd.ms-excel" in self.content_type:
                name = name + ".xlsx"
            elif "application/vnd.openxmlformats-officedocument.wordprocessingml.document" in self.content_type:
                name = name + ".docx"
            elif "application/msword" in self.content_type:
                name = name + ".doc"
            elif "application/msexcel" in self.content_type:
                name = name + ".xls"
            elif "text/xml" in self.content_type:
                name = name + ".xml"
            elif "application/pdf" in self.content_type:
                name = name + ".pdf"
            elif "text/html" in self.content_type:
                name = name + ".html"
            file = open("{}/{}".format(self.dir, name), 'w+', encoding='utf-8')
            # TODO: le fichier est mal créer et est illisible
            with file:
                file_content = base64.b64decode(self.raw_data).decode("UTF-8", "ignore")
                file.write(file_content)


# -*- coding: utf-8 -*-

import base64, os, shutil, json, datetime, unidecode, git
from src.utils import my_json
from src import settings

class Application(object):
    """
    Top node application
    """

    header = ""
    footer = ""
    token_names = ""
    git_repo_obj = ""
    git_origin = ""
    git_branch = ""
    show_menus = ""
    use_legacy_header_footer = ""
    is_hidden = ""
    help_field = ""
    is_deployed = ""
    is_managed = ""
    pub_managed = ""
    is_logo_hidden = ""

    def __init__(
        self,
        id,
        version,
        packed_id,
        displayname,
        description,
        properties,
        creation_date,
    ) -> None:
        self.id = id
        self.version = version
        self.packed_id = packed_id
        self.displayname = displayname
        self.description = description
        if hasattr(properties, "showMenus"):
            self.show_menus = properties.showMenus.cdata
        if hasattr(properties, "useLegacyHeaderFooter"):
            self.use_legacy_header_footer = properties.useLegacyHeaderFooter.cdata
        if hasattr(properties, "isHidden"):
            self.is_hidden = properties.isHidden.cdata
        if hasattr(properties, "helpField"):
            self.help_field = properties.helpField.cdata
        if hasattr(properties, "isDeployed"):
            self.is_deployed = properties.isDeployed.cdata
        if hasattr(properties, "isManaged"):
            self.is_managed = properties.isManaged.cdata
        if hasattr(properties, "header"):
            self.header = base64.b64decode(properties.header.cdata)
        if hasattr(properties, "pubManaged"):
            self.pub_managed = properties.pubManaged.cdata
        if hasattr(properties, "isLogoHidden"):
            self.is_logo_hidden = properties.isLogoHidden.cdata
        if hasattr(properties, "dependentDefs"):
            self.dependent_defs = properties.dependentDefs.cdata
        if hasattr(properties, "footer"):
            self.footer = base64.b64decode(properties.footer.cdata)
        self.creation_date = creation_date
        
    def version(self) -> str:
        return self.version
        

    def to_file(self) -> None:
        """
        Write in files application's parameters
        - README.md -> technical informations
        - header.js -> header
        - footer.js -> footer
        :return:
        """
        readme = open("{}/README.md".format(settings.APP_DIR), "w+", encoding="utf-8")
        with readme:
            # nom de l'src
            readme.write("# {}\n".format(self.displayname))
            readme.write("version {}\n\n".format(self.version))
            # Description
            readme.write("## Description\n")
            readme.write(self.description)
            # Informations
            readme.write("\n\n## Informations\n")
            readme.write("- id : {}\n".format(self.id))
            readme.write("- packedId : {}\n".format(self.packed_id))
            readme.write("\nInfos déploiement :\n")
            readme.write(
                "- [{}] showMenu \n".format("X" if self.show_menus == "true" else " ")
            )
            readme.write(
                "- [{}] useLegacyHeaderFooter\n".format(
                    "X" if self.use_legacy_header_footer == "true" else " "
                )
            )
            readme.write(
                "- [{}] isHidden\n".format("X" if self.is_hidden == "true" else " ")
            )
            readme.write(
                "- [{}] helpField\n".format("X" if self.help_field == "true" else " ")
            )
            readme.write(
                "- [{}] isDeployed\n".format("X" if self.packed_id == "true" else " ")
            )

        if self.header:
            header = open(
                "{}/header.html".format(settings.APP_DIR), "w+", encoding="utf-8"
            )
            with header:
                header.write(self.header.decode("utf-8"))

        if self.footer:
            footer = open(
                "{}/footer.html".format(settings.APP_DIR), "w+", encoding="utf-8"
            )
            with footer:
                footer.write(self.footer.decode("utf-8"))
                


# -*- coding: utf-8 -*-
import os, re

from src import settings
from. webpage import WebPage
from src.utils import split_tool
from src import settings

def creates_web_pages(webpages):
    # pages
    for webpage in webpages:
        new_webpagedef = WebPageDef(webpage)
        new_webpagedef.to_file()

class WebPageDef(object):
    """"

    """

    properties = ""
    webpages = ""

    def __init__(self, page, portal = ""):
        self.id = page["id"]
        self.orig_id = page["origId"]
        self.is_portal = page["isPortal"]
        self.obj_def_id = page["objDefId"]
        self.page_type = page["pageType"]
        self.name = page.PageDefName.cdata
        self.jsp = page.JspName.cdata
        if hasattr(page, "Props"):
            self.properties = page.Props.__dict__
        if hasattr(page, "WebPages"):
            if hasattr(page.WebPages, "WebPage"):
                self.webpages = page.WebPages.WebPage
        dirname = split_tool.split_name(self.name, 50)
        if portal:
            self.page_directory = "{}/pages/{}".format(portal, dirname)
        else:
            self.page_directory = "{}/pages/{}".format(settings.APP_DIR, dirname)


    def to_file(self):
        if not os.path.exists(self.page_directory.encode('utf-8')):
            os.makedirs(self.page_directory.encode('utf-8'))

        # fichier de propriétés
        with open("{}/PROPERTIES.md".format(self.page_directory), 'w+', encoding='utf-8') as file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))

            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- isPortal : {}\n".format(self.is_portal))
            file.write("- objDefId : {}\n".format(self.obj_def_id))
            file.write("- pageType : {}\n".format(self.page_type))
            file.write("- jsp : {}\n".format(self.jsp))

            # Propriétés
            if self.properties:
                for el in self.properties["children"]:
                    file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))

            if self.webpages:
                for webpage in self.webpages:
                    new_webpage = WebPage(webpage, self.page_directory)
                    new_webpage.to_file()




#!/usr/bin/python3

import getopt
import os
import sys

from .processor import parse_xml
from .utils.validator import check_xml
from sys import exit
from .__version__ import __version__, show_version
from . import settings
from . import usage

this_dir, this_filename = os.path.split(__file__)


def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]

    inputfile = ""
    opts = ""
    git_repo = ""

    try:
        opts, args = getopt.getopt(
            argv,
            "hV:f:o:n:g:c",
            [
                "help",
                "version",
                "file=",
                "name=",
                "git-repository-ssh=",
                "out-folder=",
                "customer=",
            ],
        )
    except getopt.GetoptError:
        usage.usage()
        exit(2)

    settings.init()

    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            exit()
        elif opt in ("-V", "--version"):
            show_version()
            exit()
        elif opt in ("-f", "--file"):
            inputfile = arg
        elif opt in ("-c", "--customer-id"):
            settings.CUSTOMER = arg
        elif opt in ("-o", "--out-folder"):
            settings.FOLDER = arg
        elif opt in ("-n", "--name"):
            settings.APP_NAME = arg
        elif opt in ("-g", "--git-repository-ssh"):
            settings.GIT_REPO = arg
    settings.SRC_DIR = this_dir

    if inputfile == "":
        print("le paramètre -f est obligatoire")
        usage.usage()
        exit(2)
    if settings.APP_NAME == "":
        print("le paramètre -n est obligatoire")
        usage.usage()
        exit(2)

    settings.APP_DIR = "{}/{}".format(settings.FOLDER, settings.APP_NAME)
    check_xml(inputfile)
    parse_xml(inputfile)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from .base import BaseField


class LongField(BaseField):
    """ Classe pour les champs de type File
            - Infos
            - DisplayLabel
            - Description
            - traductions
            - propriétés (liste d'éléments Props)
        """

    def to_file(self):
        super().to_file()

        with open("{}/PROPERTIES.md".format(self.dir), 'a+', encoding='utf-8') as file:
            file.write("## List Item\n")
            file.write("|id|origId|orderNo|source|name|code|mainItemId|isDefault|\n")
            file.write("|---|---|---|---|---|---|---|---|\n")
            for list_item in self.list_items:
                file.write("|{}|{}|{}|{}|{}|{}|{}|{}|\n".format(
                    list_item["id"],
                    list_item["origId"],
                    list_item["orderNo"],
                    list_item["source"],
                    list_item["name"],
                    list_item["code"],
                    list_item["mainItemId"],
                    list_item["isDefault"]
                ))



# process.py

import os
from src.models._base_def import AField

def creates_processes(processes, obj_dir) -> None:
    path = "{}/processes".format(obj_dir)
    if not os.path.exists(path.encode("utf-8")):
        os.makedirs(path.encode("utf-8"))
    for process in processes:
        Process(process, path).to_file()


class Process(AField):
    """ """

    translations = ""
    workflow = ""

    def __init__(self, data, path):
        super().__init__(data, path)

        self.order = data["orderNo"]
        self.obj_type_id = data["objTypeId"]
        self.def_status_id = data["defStatusId"]

    def to_file(self):
        super().to_file()

        file = open("{}/PROPETIES.md".format(self.dir), "w+", encoding="utf-8")
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- orderNo : {}\n".format(self.order))
            file.write("- objDefId : {}\n".format(self.obj_def_id))
            file.write("- objTypeId : {}\n".format(self.obj_type_id))
            file.write("- defStatusId : {}\n".format(self.def_status_id))



# base.py
import base64
from PIL import Image
from io import BytesIO

from src.models._base_def import AField


class BaseField(AField):
    """Classe générique pour définir un champ d'un objet
    - Infos
    - DisplayLabel
    - Description
    - traductions
    - propriétés (liste d'éléments Props)
    """

    list_items = ""
    validation_script = ""
    formula_script = ""
    group_name = ""
    shared_image_name = ""
    shared_image_data = ""

    def __init__(self, data, path) -> None:
        super().__init__(data, path)

        self.column_name = data["columnName"]
        self.field_name = data["fieldName"]
        self.data_class_name = data["dataClassName"]
        self.ui_class_name = data["uiClassName"].split(".")[-1]
        self.is_required = data["isRequired"]
        self.is_read_only = data["isReadOnly"]
        self.is_text_indexable = data["isTextIndexable"]
        self.is_system = data["isSystem"]
        self.is_auditable = data["isAuditable"]
        self.has_permissions = data["hasPermissions"]
        self.max_length = data["maxLength"]
        self.dir = "{}/{}/{}".format(path, self.ui_class_name, self.field_name)

        if hasattr(data, "groupeName"):
            self.group_name = data["groupName"]

        if hasattr(data, "ListItems"):
            self.list_items = data.ListItems.ListItem

        if hasattr(data, "ImageData"):
            self.shared_image_data = data.ImageData.cdata

    def to_file(self) -> None:
        super().to_file()
        with open("{}/PROPERTIES.md".format(self.dir), "w+", encoding="utf-8") as file:
            file.write("## {}\n".format(self.name))
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDef : {}\n".format(self.obj_def_id))
            file.write("- columnName : {}\n".format(self.column_name))
            file.write("- fieldName : {}\n".format(self.field_name))
            if self.group_name:
                file.write("- groupName : {}\n".format(self.group_name))
            file.write("- dataClassName : {}\n".format(self.data_class_name))
            file.write("- uidClassName : {}\n".format(self.ui_class_name))
            file.write("- isRequired : {}\n".format(self.is_required))
            file.write("- isReadOnly : {}\n".format(self.is_read_only))
            file.write("- isTextIndexable : {}\n".format(self.is_text_indexable))
            file.write("- isSystem : {}\n".format(self.is_system))
            file.write("- isAuditable : {}\n".format(self.is_auditable))
            file.write("- hasPermissions : {}\n".format(self.has_permissions))
            file.write("- maxLength : {}\n".format(self.max_length))

            # Translations
            if self.translations:
                file.write("## Traductions\n")
                file.write("|code|nom|texte|\n")
                file.write("|---|---|---|\n")
                for trans in self.translations:
                    file.write(
                        "|{}|{}|{}|\n".format(
                            trans["langCode"], trans["fieldName"], trans["text"]
                        )
                    )

    def set_properties(self):
        if self.properties:
            with open(
                "{}/PROPERTIES.md".format(self.dir), "a+", encoding="utf-8"
            ) as file:
                file.write("## Propriétés\n")
                for el in self.properties["children"]:
                    if el.__dict__["_name"] == "templateB64":
                        with open(
                            "{}/template.html".format(self.dir), "w+", encoding="utf-8"
                        ) as template:
                            template.write(
                                base64.b64decode(el.__dict__["cdata"]).decode("utf-8")
                            )
                    elif el.__dict__["_name"] == "formulaB64":
                        with open(
                            "{}/formula.js".format(self.dir), "w+", encoding="utf-8"
                        ) as formula:
                            formula.write(
                                base64.b64decode(el.__dict__["cdata"]).decode("utf-8")
                            )
                    elif el.__dict__["_name"] == "linkTemplB64":
                        with open(
                            "{}/link.html".format(self.dir), "w+", encoding="utf-8"
                        ) as link:
                            link.write(
                                base64.b64decode(el.__dict__["cdata"]).decode("utf-8")
                            )
                    elif el.__dict__["_name"] == "sharedImage":
                        self.shared_image_name = "{}.{}".format(
                            self.name, el.__dict__["cdata"].split(".")[-1]
                        )
                        im = Image.open(
                            BytesIO(base64.b64decode(self.shared_image_data))
                        )
                        im.save(
                            "{}/{}".format(self.dir, self.shared_image_name),
                            el.__dict__["cdata"].split(".")[-1],
                        )
                    else:
                        file.write(
                            "- {} : {}\n".format(
                                el.__dict__["_name"], el.__dict__["cdata"]
                            )
                        )



# field.py

import os
from .long import LongField
from .longarr import LongArrField
from .base import BaseField

def creates_fields(fields, obj_dir):
    # créer un dossier fields + nouveaux fields
    path = "{}/fields".format(obj_dir)
    if not os.path.exists(path.encode('utf-8')):
        os.makedirs(path.encode('utf-8'))
    for field_data in fields:
        # FieldLong
        if "FieldLong" in field_data["dataClassName"]:
            field = LongField(field_data, path)
        # FieldLongArr
        elif "FieldLongArr" in field_data["dataClassName"]:
            field = LongArrField(field_data, path)
        else:
            field = BaseField(field_data, path)
        if field != "":
            field.to_file()
            field.set_properties()



# processor.py

import shutil
import time
from src import settings
from src.utils import my_json
from src.git_tools.git_repo import GitRepo
import untangle, os, json
from src.models.application import Application


def parse_xml(xml) -> None:
    time_start = time.perf_counter()
    print("processing {}".format(xml))

    app_tree = untangle.parse(xml)
    date_creation_fichier = os.path.getmtime(xml)
    json_path = os.path.join(settings.SRC_DIR, "utils", "id_dict.json")
    file_id = json.load(open(json_path))

    # objets dépendants -> ajout dans la liste de référence
    if hasattr(app_tree.Application.DependentDefs, "DataObjectDef"):
        for DataObjectDef in app_tree.Application.DependentDefs.DataObjectDef:
            obj = my_json.find_id_json(file_id, DataObjectDef["id"])
            if not obj:
                file_id[DataObjectDef["id"]] = DataObjectDef["origId"]

        jsontest = json.dumps(file_id)
        with open(json_path, "w", encoding="utf-8") as f:
            f.write(jsontest)

        del jsontest, f, file_id

    props = ""
    description = ""
    
    if hasattr(app_tree.Application, "Props"):
        props = app_tree.Application.Props

    if hasattr(app_tree.Application, "Description"):
        description = app_tree.Application.Description.cdata
    
    app = Application(
        app_tree.Application["id"],
        app_tree.Application["version"],
        app_tree.Application["packedId"],
        app_tree.Application.DisplayName.cdata,
        description,
        props,
        date_creation_fichier,
    )

    git_object = GitRepo(date_creation_fichier, app.version)
    
    if settings.GIT_REPO:
        git_object.del_files()
        git_object.check_git_exists()
        git_object.git_set_branch(settings.CUSTOMER)
    
    # creates Application basics
    app.to_file()
    
    del app

    # creates application Tokens
    if hasattr(app_tree.Application, "Props"):
        if hasattr(props, "tokenNames"):
            from src.models.token import creates_tokens
            creates_tokens(props.tokenNames.cdata)

    # creates custom logo
    if hasattr(app_tree.Application, "CustomLogo"):
        from src.models.logo import creates_logo
        creates_logo(app_tree.Application.CustomLogo.cdata, app_tree.Application.Props.appLogo.cdata)
    
    # creates application Dependent Object
    if hasattr(app_tree.Application, "DependentDefs"):
        if hasattr(app_tree.Application.DependentDefs, "DataObjectDef"):
            from src.models.dependent import creates_dependents
            creates_dependents(app_tree.Application.DependentDefs.DataObjectDef)
    
    # creates menus
    if hasattr(app_tree.Application, "Menus"):
        if hasattr(app_tree.Application.Menus, "Menu"):
            from src.models.menu import creates_menus
            creates_menus(app_tree.Application.Menus.Menu)
    
    # creates hosted files 
    if hasattr(app_tree.Application, "HostedFiles"):
        from src.models.hosted_file import create_hosted_files
        create_hosted_files(app_tree.Application.HostedFiles.HostedFile)
            
    # creates Web pages
    if hasattr(app_tree.Application, "WebPageDefs"):
        from src.models.webpagedef import creates_web_pages
        creates_web_pages(app_tree.Application.WebPageDefs.WebPageDef)

    # creates Portals
    if hasattr(app_tree.Application, "Portals"):
        if hasattr(app_tree.Application.Portals, "WebSite"):
            from src.models.portal import creates_portals
            creates_portals(app_tree.Application.Portals.WebSite)

    # creates Seed Records
    if hasattr(app_tree.Application, "SeedRecords"):
        from src.models.seedrecord import creates_seeds
        creates_seeds(app_tree.Application.SeedRecords.DataObject)

    # creates Batch jobs
    if hasattr(app_tree.Application, "BatchJobs"):
        if hasattr(app_tree.Application.BatchJobs, "Event"):
           from src.models.batch_job import creates_batchs
           creates_batchs(app_tree.Application.BatchJobs.Event)
            
    # creates Custom methods
    if hasattr(app_tree.Application, "CustomMethods"):
        if hasattr(app_tree.Application.CustomMethods, "CustomMethod"):
            from src.models.custom_method import creates_custom_methods
            creates_custom_methods(app_tree.Application.CustomMethods.CustomMethod)
    
    # creates Custom Reports
    if hasattr(app_tree.Application, "CustomReports"):
        if hasattr(app_tree.Application.CustomReports, "CustomReport"):
            pass
            # TODO: creates_custom_reports(Application.CustomReports.CustomReport)
           
    # creates Permissions
    if hasattr(app_tree.Application, "Permissions"):
        if hasattr(app_tree.Application.Permissions, "Permission"):
            from src.models.permission import creates_permissions
            creates_permissions(app_tree.Application.Permissions.Permission)
           
    # creates RolePages
    if hasattr(app_tree.Application, "RolePages"):
        if hasattr(app_tree.Application.RolePages, "RolePage"):
            from src.models.role_page import creates_role_pages
            creates_role_pages(app_tree.Application.RolePages.RolePage)

    # creates Translations
    if hasattr(app_tree.Application, "Translations"):
        if hasattr(app_tree.Application.Translations, "Translation"):
            from src.models.translation import creates_translations
            creates_translations(app_tree.Application.Translations.Translation)

    # creates  Objects in the end
    if hasattr(app_tree.Application, "DataObjectDefs"):
        if hasattr(app_tree.Application.DataObjectDefs, "DataObjectDef"):
            from src.models.object_def.object import creates_object_defs
            creates_object_defs(app_tree.Application.DataObjectDefs.DataObjectDef)
            
    text_file = "{}/app.xml".format(settings.APP_DIR.rstrip("/"))
    shutil.copy(xml, text_file)

    if settings.GIT_REPO:
        git_object.commit_tag_push()
        git_object.git_set_branch()
        git_object.git_push()
        pass

    time_elapsed = time.perf_counter() - time_start

    print(
        "application '{}' generated in {}s".format(
            app_tree.Application.DisplayName.cdata, round(time_elapsed, 2)
        )
    )



def find_in_json(data, obj_id):
    try:
        return data[obj_id]
    except KeyError:
        return ""


def find_id_json(data, obj_id):
    try:
        return data[obj_id]
    except KeyError:
        return ""



# permission.py

from src import settings

def creates_translations(translations, dir = ""):
    if dir:
        final_dir = "{}/TRANSLATIONS.md".format(dir)
    else :
        final_dir = "{}/TRANSLATIONS.md".format(settings.APP_DIR)
    t = open(final_dir, "w+", encoding="utf-8")
    with t:
        t.write("\n## Translations :\n")
        
        t.write("|fieldName|langCode|text|\n|---|---|---|\n")
        for x in translations:
                t.write("|{}|{}|{}|\n".format(x["fieldName"], x["langCode"], x["text"]))


# validator.py

from lxml import etree
import os


def check_xml(xml_to_check):
    # parse xml
    try:
        if not os.path.isfile(xml_to_check):
            raise FileNotFoundError

        with open(xml_to_check, "r", encoding="utf-8") as file:
            xml = file.read()
            xml = bytes(bytearray(xml, encoding="utf-8"))
            etree.XML(xml)

    # check for FileNotFoundError
    except FileNotFoundError:
        print("Can't find file {}".format(xml_to_check))
        exit(2)

    # check for file IO error
    except IOError as err:
        print("Invalid File")
        with open("error.log", "w", encoding="utf-8") as error_log_file:
            error_log_file.write(str(err))

        exit(2)

    # check for XML syntax errors
    except etree.XMLSyntaxError as err:
        print("XML Syntax Error, see error.log")
        with open("error.log", "w", encoding="utf-8") as error_log_file:
            error_log_file.write(str(err))
        exit(2)

    except Exception as err:
        print("Unknown error, see error.log")
        with open("error.log", "w", encoding="utf-8") as error_log_file:
            error_log_file.write(str(err))
        exit(2)



import datetime, os, json, unidecode, git
import shutil
from git import Repo, Actor, Commit
from src import settings

class GitRepo(object):
    
    repo = None
    origin = None
    
    def __init__(self, creation_time, version) -> None:
        print("version is {}".format(version))
        self.creation_time = creation_time
        self.version = version
        
    def del_files(self) -> None:
        if not os.path.exists(settings.APP_DIR.encode("utf-8")):
            os.makedirs(settings.APP_DIR.encode("utf-8"))
            
        # delete all files and folder to make git tracking easyest
        else:
            for f in os.listdir(settings.APP_DIR):
                # if not f.startswith('.'):
                if os.path.isfile("{}/{}".format(settings.APP_DIR, f)):
                    os.remove("{}/{}".format(settings.APP_DIR, f))
                else:
                    shutil.rmtree("{}/{}".format(settings.APP_DIR, f).encode("utf-8"))


    def check_git_exists(self) -> None:
        if not os.path.exists(settings.APP_DIR + "/.git"):
            # os.makedirs(self.app_directory)
            self.repo = Repo.init(settings.APP_DIR)
            self.origin = self.repo.create_remote(
                "origin", url=settings.GIT_REPO
            )
            # self.git_repo_obj\
            #     .create_head('master', self.git_origin.refs.master)\
            #     .set_tracking_branch(self.git_origin.refs.master)\
            #     .checkout()
        else:
            self.repo = Repo(settings.APP_DIR)
            self.origin = self.repo.remote("origin")
            self.origin
        self.origin.fetch()
        self.repo.remotes.origin.push("--all")

    def git_set_branch(self, customer_id=None) -> None:
        customer_name = "master"
        if customer_id:
            file_id = json.load(open("./src/utils/customer.json"))

            for i in file_id:
                if i["id"] == int(customer_id):
                    customer_name = (
                        i["name"].lower().replace(" ", "_").replace("-", "_")
                    )
                    break
            customer_name = unidecode.unidecode(customer_name)
        try:
            self.repo.git.checkout(customer_name)
        except git.exc.GitCommandError:
            self.repo.git.checkout("-b", customer_name)
        self.git_branch = customer_name

    def git_push(self) -> None:
        try:
            print("push")
            self.origin.push()
        except Exception as e:
            print('Some error occured while pushing the code : ' + str(e)) 

    def commit_tag_push(self) -> None:
        repo = self.repo
        # add all changes
        repo.git.add(A=True)
        # tree = repo.index.write_tree()

        # create commit with xml file's date

        # Committer and Author
        cr = repo.config_reader()
        committer = Actor.committer(cr)
        author = Actor.author(cr)

        date_creation = datetime.datetime.fromtimestamp(self.creation_time)
        date_creation_git = date_creation.replace(microsecond=0).isoformat()

        # offset = altzone
        # author_time, author_offset = date_creation, offset
        # committer_time, committer_offset = date_creation, offset

        message = "Commit automatique du " + date_creation.strftime("%d/%m/%Y")

        os.environ["GIT_AUTHOR_DATE"] = str(date_creation)
        os.environ["GIT_COMMITTER_DATE"] = str(date_creation)

        # Do the commit thing.
        commit = repo.index.commit(
            message,
            author=author,
            committer=committer,
            commit_date=date_creation_git,
            author_date=date_creation_git,
        )

        # Create TAG if version changed
        repo.remotes.origin.push(self.git_branch)
        tags = repo.tags
        if self.version not in tags:
            repo.create_tag(
                self.version, message='Automatic tag "{0}"'.format(self.version)
            )
            repo.remotes.origin.push(self.version)

        # Do the push
        repo.remotes.origin.push(self.git_branch)


# hosted_file.py

from src import settings
import os, base64

def create_hosted_files(files):
    ### HOSTED FILES
    hosted_files_dir = "{}/hosted_files".format(settings.APP_DIR)
    for file in files:
        if not os.path.exists(hosted_files_dir.encode("utf-8")):
            os.mkdir(hosted_files_dir.encode("utf-8"))
            os.mkdir("{}/css".format(hosted_files_dir).encode("utf-8"))
            os.mkdir("{}/js".format(hosted_files_dir).encode("utf-8"))
            os.mkdir("{}/images".format(hosted_files_dir).encode("utf-8"))

        folder = ""
        ext = ""
        is_image = False

        if file["contentType"] == "text/css":
            folder = ext = "css"
        elif file["contentType"] == "application/javascript":
            folder = ext = "js"
        elif file["contentType"] == "application/x-javascript":
            folder = "js"
            ext = "min.js"
        elif file["contentType"] == "image/png":
            folder = "images"
            ext = "png"
            is_image = True
        elif file["contentType"] == "image/jpeg":
            folder = "images"
            ext = "jpg"
            is_image = True

        if folder and not is_image:
            with open(
                "{}/{}/#HOSTED_FILE.{}-{}.{}".format(
                    hosted_files_dir,
                    folder,
                    file["origId"],
                    file.DisplayName.cdata.replace("/", "-")
                    .replace(" ", "_")
                    .replace("\\", "-")
                    .replace("&", "")
                    .replace("!", ""),
                    ext,
                ),
                "w+",
                encoding="utf-8",
            ) as readme:
                file_content = base64.b64decode(file.RawData.cdata).decode(
                    "iso8859_15", "ignore"
                )
                readme.write(file_content)
        elif folder and is_image:
            with open(
                "{}/{}/#HOSTED_FILE.{}-{}.{}".format(
                    hosted_files_dir,
                    folder,
                    file["origId"],
                    file.DisplayName.cdata.replace("/", "-")
                    .replace(" ", "_")
                    .replace("\\", "-")
                    .replace("&", "")
                    .replace("!", ""),
                    ext,
                ),
                "wb",
            ) as readme:
                if hasattr(file, "RawData"):
                    file_content = base64.b64decode(file.RawData.cdata)
                    readme.write(file_content)


# -*- coding: utf-8 -*-
import base64, os

# TODO: cell js en fonction du type de cell
class Cell(object):
    """

    """

    properties = ""
    script = ""

    def __init__(self, cell, section_directory):
        self.id = cell["id"]
        self.orig_id = cell["origId"]
        self.section_id = cell["sectionId"]
        self.order_no = cell["orderNo"]
        self.alignment = cell["alignment"]
        self.field_id = cell["fieldId"]
        self.cell_class_name = cell["cellClassName"].split('.')[-1]
        self.cell_directory = "{}".format(section_directory)
        if hasattr(cell, "Props"):
            self.properties = cell.Props.__dict__
        if hasattr(cell, "TextB64"):
            self.script = cell.TextB64.cdata

    def to_file(self):
        # fichier de propriétés
        # if not os.path.exists(self.cell_directory):
        #     os.makedirs(self.cell_directory)
        with open("{}/_cell_{}.md".format(self.cell_directory, self.order_no), 'w+', encoding='utf-8') as file:
            # nom de l'objet
            file.write("# {}\n".format(self.cell_class_name))

            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- sectionId : {}\n".format(self.section_id))
            file.write("- orderNo : {}\n".format(self.order_no))
            file.write("- alignment : {}\n".format(self.alignment))
            file.write("- fieldId : {}\n".format(self.field_id))
            file.write("- cellClassName : {}\n".format(self.cell_class_name))

            # Propriétés
            if self.properties:
                for el in self.properties["children"]:
                    file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))
        decoded = base64.b64decode(self.script).decode("utf-8")
        if self.script and self.cell_class_name == "TemplateHTML":
            self.create_code("html", decoded)
        if self.script and self.cell_class_name == "ScriptControl": 
            if "<script" in decoded or "<style" in decoded:
                self.create_code("js", decoded)
            else:
                self.create_code("html", decoded)

    def create_code(self, ext, decoded):
        with open("{}/_cell_{}.{}".format(self.cell_directory, self.order_no, ext), 'w+', encoding='utf-8') as script:
            script.write(decoded)



# base_field.py

import base64
import binascii
import os
from src.models.translation import creates_translations
from src.models.object_def.workflow import creates_workflow
from src.utils import split_tool


class AField(object):
    """ "
    Blueprint for field

    Contains dir, id, origId, orderNo, objDefId, name, properties, ConditionB64, ScriptB64, Translation, Workflow
        ValidationB64
    Creates folder if not exists
    """

    name = None
    properties = None
    condition_base64 = None
    script_base64 = None
    validation_base64 = None
    translations = None
    workflow = None
    columns = None

    def __init__(self, data, path) -> None:
        self.id = data["id"]
        self.orig_id = data["origId"]
        self.obj_def_id = data["objDefId"]
        if hasattr(data, "objDef"):
            self.obj_def_id = data["objDef"]
        self.name = data["name"]
        if hasattr(data, "DisplayName"):
            self.name = data.DisplayName.cdata
        elif hasattr(data, "DisplayLabel"):
            self.name = data.DisplayLabel.cdata
        elif hasattr(data, "ViewName"):
            self.name = data.ViewName.cdata
        elif hasattr(data, "ChartName"):
            self.name = data.ChartName.cdata
        elif hasattr(data, "MapColumns"):
            self.name = data["mapName"]
        # if hasattr(data, "name"):

        self.dir = "{}/{}".format(path, split_tool.split_name(self.name))

        if hasattr(data, "Description"):
            self.description = data.Description.cdata

        if hasattr(data, "Props"):
            self.properties = data.Props.__dict__

        if hasattr(data, "ConditionB64"):
            self.condition_base64 = data.ConditionB64.cdata

        if hasattr(data, "ScriptB64"):
            self.scriscript_base64pt = data.ScriptB64.cdata

        if hasattr(data, "Translations"):
            if hasattr(data.Translations, "Translation"):
                self.translations = data.Translations.Translation

        if hasattr(data, "Workflow"):
            if hasattr(data.Workflow, "Node"):
                self.workflow = data.Workflow.Node

        if hasattr(data, "ValidationScriptB64"):
            self.validation_base64 = data.ValidationScriptB64.cdata

        if hasattr(data, "MapColumns"):
            if hasattr(data.MapColumns, "MapColumn"):
                self.columns = data.MapColumns.MapColumn

    def to_file(self) -> None:
        if not os.path.exists(self.dir.encode("utf-8")):
            os.makedirs(self.dir.encode("utf-8"))

        if self.translations:
            creates_translations(self.translations, self.dir)

        if self.workflow:
            creates_workflow(self.workflow, self.dir)

        if self.condition_base64:
            file = open("{}/condition.js".format(self.dir), "w+", encoding="utf-8")
            with file:
                file_content = base64.b64decode(self.condition_base64).decode(
                    "UTF-8", "ignore"
                )
                file.write(file_content)

        if self.script_base64:
            file = open("{}/script.js".format(self.dir), "w+", encoding="utf-8")
            with file:
                file_content = base64.b64decode(self.script_base64).decode(
                    "UTF-8", "ignore"
                )
                file.write(file_content)

        if self.validation_base64:
            file = open(
                "{}/validation_script.js".format(self.dir), "w+", encoding="utf-8"
            )
            with file:
                file_content = base64.b64decode(self.validation_base64).decode(
                    "UTF-8", "ignore"
                )
                file.write(file_content)

    def template(self, text):
        template = open("{}/template.js".format(self.dir), "w+", encoding="utf-8")
        with template:
            try:
                template_content = base64.b64decode(text).decode("UTF-8", "ignore")
                template.write(template_content)
            except binascii.Error:
                template.write(text)
            except ValueError:
                template.write(text)



# object.py
import json
import os
from src import settings
from src.models.batch_job import creates_batchs
from src.models.object_def.action import creates_actions
from src.models.object_def.button import creates_buttons
from src.models.object_def.chart import creates_charts
from src.models.object_def.conversion import creates_conversions
from src.models.object_def.event import creates_events
from src.models.object_def.field import creates_fields
from src.models.object_def.process import creates_processes
from src.models.object_def.status import creates_statuses
from src.models.object_def.template import creates_templates
from src.models.object_def.view import creates_views
from src.models.translation import creates_translations
from src.utils import my_json, split_tool
from .relationship import creates_relationships


def creates_object_defs(objects):
    # objets de l'application
    for appObject in objects:
        new_object = Object(appObject)
        new_object.create()


class Object(object):

    """
    définition d'un object
    comporte liste de 'fields'
    """

    description = ""
    is_managed = ""
    def_process = ""
    audit_view = ""
    audit_create = ""
    audit_delete = ""
    audit_edit = ""
    enable_reports = ""
    show_tags = ""
    goog_synch = ""
    translations = ""
    relations = ""
    views = ""
    events = ""
    menus = ""
    processes = ""
    statuses = ""
    actions = ""
    conversions = ""
    buttons = ""
    templates = ""
    charts = ""
    is_localized = ""

    def __init__(self, data):
        self.id = data["id"]
        self.orig_id = data["origId"]
        self.name = split_tool.split_name(data.SingularName.cdata)
        self.def_name = data["objDefName"]
        self.is_system = data["isSystem"]
        self.is_auditable = data["isAuditable"]
        self.is_viewable = data["isViewable"]
        self.is_flagdable = data["isFlaggable"]
        self.is_dependent = data["isDependent"]
        self.is_deployed = data["isDeployed"]
        self.obj_directory = "{}/objects/{}".format(settings.APP_DIR, self.def_name)
        self.data = data

        # id -> origin id
        json_path = os.path.join(settings.SRC_DIR, "utils", "id_dict.json")
        file_id = json.load(open(json_path))
        obj = my_json.find_id_json(file_id, self.id)
        if not obj:
            file_id[self.id] = self.orig_id

        jsontest = json.dumps(file_id)
        with open(json_path, "w", encoding="utf-8") as f:
            f.write(jsontest)

        # obj_dict
        json_dict = os.path.join(settings.SRC_DIR, "utils", "objects_dict.json")
        file_dict = json.load(open(json_dict))
        obj = my_json.find_id_json(file_dict, self.id)
        if not obj:
            inner_data = {
                "origId": self.orig_id,
                "name": self.name,
                "defName": self.def_name,
            }
            file_dict[self.id] = inner_data
            file_dict[self.orig_id] = inner_data

        jsontest = json.dumps(file_dict)
        with open(json_dict, "w", encoding="utf-8") as f:
            f.write(jsontest)

    def create(self):
        """
        Ecrit dans des fichiers les paramètres de l'obejt
        - PROPERTIES.md avec les infos techniques
        - RELATIONSHIP.md avec les relations
        - VIEWS.md avec les définitions des view
        - TRANSLATIONS.yml pour les traductions de l'objet
        - un dossier par champ dans le dossier fields
        - un dossier par déclencheur dans le dossier triggers
        :return:

        """
        if not os.path.exists(self.obj_directory.encode("utf-8")):
            os.makedirs(self.obj_directory.encode("utf-8"))

        data = self.data
        dir = self.obj_directory

        if hasattr(data, "Description"):
            self.description = data.Description.cdata
        if hasattr(data, "Props"):
            if hasattr(data.Props, "isManaged"):
                self.is_managed = data.Props.isManaged.cdata
            if hasattr(data.Props, "defProcess"):
                self.def_process = data.Props.defProcess.cdata
            if hasattr(data.Props, "auditView"):
                self.audit_view = data.Props.auditView.cdata
            if hasattr(data.Props, "auditCreate"):
                self.audit_create = data.Props.auditCreate.cdata
            if hasattr(data.Props, "auditDelete"):
                self.audit_delete = data.Props.auditDelete.cdata
            if hasattr(data.Props, "auditEdit"):
                self.audit_edit = data.Props.auditEdit.cdata
            if hasattr(data.Props, "enableReports"):
                self.enable_reports = data.Props.enableReports.cdata
            if hasattr(data.Props, "showTags"):
                self.show_tags = data.Props.showTags.cdata
            if hasattr(data.Props, "googSynch"):
                self.goog_synch = data.Props.googSynch.cdata
            if hasattr(data.Props, "deleteFormulaBase64"):
                self.delete_formula = data.Props.deleteFormulaBase64.cdata
            if hasattr(data.Props, "isLocalized"):
                self.is_localized = data.Props.isLocalized.cdata

        if hasattr(data, "Translations"):
            if hasattr(data.Translations, "Translation"):
                creates_translations(data.Translations.Translation, dir)

        if hasattr(data, "RelationshipDefs"):
            if hasattr(data.RelationshipDefs, "RelationshipDef"):
                creates_relationships(data.RelationshipDefs.RelationshipDef, dir)

        if hasattr(data, "ListViews"):
            creates_views(data.ListViews.ListView, dir)

        if hasattr(data, "Events"):
            creates_events(data.Events.Event, dir)

        if hasattr(data, "Actions"):
            creates_actions(data.Actions.Action, dir)

        if hasattr(data, "Statuses"):
            creates_statuses(data.Statuses.Status, dir)

        if hasattr(data, "Processes"):
            creates_processes(data.Processes.Process, dir)

        if hasattr(data, "Buttons"):
            creates_buttons(data.Buttons.Button, dir)

        if hasattr(data, "ConversionMaps"):
            if hasattr(data.ConversionMaps, "ImportMap"):
                creates_conversions(data.ConversionMaps.ImportMap, dir)

        if hasattr(data, "Charts"):
            creates_charts(data.Charts.Chart, dir)

        if hasattr(data, "Templates"):
            creates_templates(data.Templates.Template, dir)

        creates_fields(data.DataFieldDefs.DataFieldDef, dir)

    def creates_properties(self):
        # fichier de propriétés
        with open(
            "{}/PROPERTIES.md".format(self.obj_directory), "w+", encoding="utf-8"
        ) as file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Description
            if self.description:
                file.write("## Description\n")
                file.write(self.description)
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDef : {}\n".format(self.def_name))
            # Déploiement
            file.write("\nInfos déploiement :\n")
            file.write(
                "- [{}] isSystem \n".format("X" if self.is_system == "true" else " ")
            )
            file.write(
                "- [{}] isAuditable\n".format(
                    "X" if self.is_auditable == "true" else " "
                )
            )
            file.write(
                "- [{}] isViewable\n".format("X" if self.is_viewable == "true" else " ")
            )
            file.write(
                "- [{}] isFlaggable\n".format(
                    "X" if self.is_flagdable == "true" else " "
                )
            )
            file.write(
                "- [{}] isDependent\n".format(
                    "X" if self.is_dependent == "true" else " "
                )
            )
            file.write(
                "- [{}] isDeployed\n".format("X" if self.is_deployed == "true" else " ")
            )
            # Propriétés
            file.write("\nPropriétés :\n")
            if self.is_managed:
                file.write(
                    "- [{}] isManaged \n".format(
                        "X" if self.is_managed == "true" else " "
                    )
                )
            if self.def_process:
                file.write(
                    "- [{}] defProcess\n".format(
                        "X" if self.def_process == "1" else " "
                    )
                )
            if self.audit_view:
                file.write(
                    "- [{}] auditView\n".format(
                        "X" if self.audit_view == "true" else " "
                    )
                )
            if self.audit_create:
                file.write(
                    "- [{}] auditCreate\n".format(
                        "X" if self.audit_create == "true" else " "
                    )
                )
            if self.audit_delete:
                file.write(
                    "- [{}] auditDelete\n".format(
                        "X" if self.audit_delete == "true" else " "
                    )
                )
            if self.audit_edit:
                file.write(
                    "- [{}] auditEdit\n".format(
                        "X" if self.audit_edit == "true" else " "
                    )
                )
            if self.enable_reports:
                file.write(
                    "- [{}] enableReports\n".format(
                        "X" if self.enable_reports == "true" else " "
                    )
                )
            if self.show_tags:
                file.write(
                    "- [{}] showTags\n".format("X" if self.show_tags == "true" else " ")
                )
            if self.goog_synch:
                file.write(
                    "- [{}] googSync\n".format(
                        "X" if self.goog_synch == "true" else " "
                    )
                )
            if self.is_localized:
                file.write(
                    "- [{}] isLocalized\n".format(
                        "X" if self.is_localized == "true" else " "
                    )
                )



# portal.py
import os, base64
from .webpagedef import WebPageDef
from src.utils import split_tool
from src import settings

def creates_portals(portals):
    # portals
    for portal in portals:
        new_portal = Portal(portal, settings.APP_DIR)
        new_portal.to_file()
        new_portal.create_web_pages()


class Portal(object):
    """
    Défini un portail Rollase
    """

    def __init__(self, portal, app_dir):
        self.id = portal["id"]
        self.orig_id = portal["origId"]
        self.is_main = portal["isMain"]
        self.name = portal.SiteName.cdata
        self.stylesheet = portal.Stylesheet.cdata
        if hasattr(portal.WebPageDefs, "Header"):
            self.header = base64.b64decode(portal.Header.cdata)
        else:
            self.header = ""
        if hasattr(portal.WebPageDefs, "Footer"):
            self.footer = base64.b64decode(portal.Footer.cdata)
        else:
            self.footer = ""
        if hasattr(portal.WebPageDefs, "WebPageDef"):
            self.webpages = portal.WebPageDefs.WebPageDef
        else:
            self.webpages = ""
        dirname = split_tool.split_name(self.name, 50)
        self.portal_dir = "{}/portals/{}".format(app_dir, dirname)
        self.properties = portal.__dict__

    def to_file(self):
        # fichier de propriétés
        if not os.path.exists(self.portal_dir.encode('utf-8')):
            os.makedirs(self.portal_dir.encode('utf-8'))
        file = open("{}/PROPERTIES.md".format(self.portal_dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- isMain : {}\n".format(self.is_main))

            # Propriétés
            file.write("\n\n## Properties\n")
            for el in self.properties["children"]:
                file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))

        header = open("{}/header.html".format(self.portal_dir), 'w+', encoding='utf-8')
        with header:
            header.write(self.header)

        footer = open("{}/footer.html".format(self.portal_dir), 'w+', encoding='utf-8')
        with footer:
            footer.write(self.footer)

        if self.webpages:
            self.create_web_pages()

    def create_web_pages(self):
        # objets de l'application
        for webpage in self.webpages:
            new_webpagedef = WebPageDef(webpage, self.portal_dir)
            new_webpagedef.to_file()



# chart.py

import os
from src.models._base_def import AField


def creates_charts(charts, obj_dir) -> None:
    path = "{}/charts".format(obj_dir)
    if not os.path.exists(path.encode("utf-8")):
        os.makedirs(path.encode("utf-8"))
    for chart in charts:
        Chart(chart, path).to_file()


class Chart(AField):
    """ "
    Chart def
    """

    def __init__(self, data, path) -> None:
        super().__init__(data, path)
        self.order = data["orderNo"]

    def to_file(self) -> None:
        super().to_file()

        file = open("{}/PROPERTIES.md".format(self.dir), "w+", encoding="utf-8")
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDefId : {}\n".format(self.obj_def_id))
            file.write("- order : {}\n".format(self.order))



# action.py

import os
from src.models._base_def import AField

def creates_actions(actions, obj_dir) -> None:
        path = "{}/actions".format(obj_dir)
        if not os.path.exists(path.encode('utf-8')):
            os.makedirs(path.encode('utf-8'))
        for action_data in actions:
            action = Action(action_data, path)
            action.to_file()

class Action(AField):
    """

    """

    def __init__(self, action, path) -> None:
        super().__init__(action, path)
        self.order = action["orderNo"]
        self.action_type = action["actionType"]
        self.order = action["orderNo"]
        self.page_id = action["pageId"]
        self.next_stauts_id = action ["nextStatusId"]
        self.is_group = action["isGroup"]
        self.obj_type_id = action["objTypeId"]
        

    def to_file(self) -> None:
        super().to_file()
        # fichier de propriétés
        if not os.path.exists(self.dir.encode('utf-8')):
            os.makedirs(self.dir.encode('utf-8'))
        file = open("{}/PROPERTIES.md".format(self.dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- orderNo : {}\n".format(self.order))
            file.write("- objDefId : {}\n".format(self.obj_type_id))
            file.write("- pageId : {}\n".format(self.page_id))
            file.write("- nextStatusId : {}\n".format(self.next_stauts_id))
            file.write("- isGroup : {}\n".format(self.is_group))
            file.write("- objTypeId : {}\n".format(self.obj_type_id))

            # Propriétés
            if self.properties:
                file.write("\n\n## Properties\n")
                for el in self.properties["children"]:
                    file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))
            



# meny.py

import os, re
from src.utils import split_tool
from src import settings

def creates_menus(menus):
    # Create a menu folder + menus
    # create a file MENUS.md with tree menu
    path = "{}/menus".format(settings.APP_DIR)
    if not os.path.exists(path.encode("utf-8")):
        os.makedirs(path.encode("utf-8"))
    menu_file = open(
        "{}/MENUS.md".format(settings.APP_DIR), "w+", encoding="utf-8"
    )
    menu_file.write("## Menus\n\n```\n.\n")
    for menu in menus:
        new_menu = Menu(menu, path)
        new_menu.to_file()
        menu_file.write(new_menu.get_menus(menus))
    menu_file.write("```\n")

class Menu(object):
    """
        Défini un menu Rollase
        """
    translations = ""
    sub_menus = ""

    def __init__(self, menu, dir):
        self.id = menu["id"]
        self.orig_id = menu["origId"]
        self.order = menu["orderNo"]
        self.obj_def = menu["objDef"]
        self.parent_id = menu["parentId"]
        self.page_def_id = menu["pageDefId"]
        self.name = menu.DisplayName.cdata
        if hasattr(menu, "Translations"):
            if hasattr(menu.Translations, "Translation"):
                self.translations = menu.Translations.Translation
        if hasattr(menu, "Menus"):
            if hasattr(menu.Menus, "Menu"):
                self.sub_menus = menu.Menus.Menu
        dirname = split_tool.split_name(self.name)
        self.menu_dir = "{}/{}".format(dir, dirname)

    def to_file(self):
        # fichier de propriétés
        if not os.path.exists(self.menu_dir.encode('utf-8')):
            os.makedirs(self.menu_dir.encode('utf-8'))
        file = open("{}/PROPERTIES.md".format(self.menu_dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDef : {}\n".format(self.obj_def))
            file.write("- parentId : {}\n".format(self.parent_id))
            file.write("- orderNo : {}\n".format(self.order))

            if self.translations:
                file.write("\n## Traductions\n")
                file.write("|code|nom|text|\n")
                file.write("|---|---|---|\n")
                for trans in self.translations:
                    file.write("|{}|{}|{}|\n".format(trans["langCode"], trans["fieldName"], trans["text"]))

            if self.sub_menus:
                file.write("\n## Sub menus\n```\n")
                file.write(".\n|-- {}\n".format(self.name))
                for sub_menu in self.sub_menus:
                    new_menu = Menu(sub_menu, self.menu_dir)
                    new_menu.to_file()
                    file.write("|   |-- {}\n".format(new_menu.name))
                file.write("```\n")

    def get_menus(self, menus):
        menu_string = ""
        if self.sub_menus:
            menu_string += "|-- {}\n".format(self.name)
            for sub_menu in self.sub_menus:
                new_menu = Menu(sub_menu, self.menu_dir)
                menu_string += "|   |-- {}\n".format(new_menu.name)
                if hasattr(new_menu, "menus"):
                    if new_menu.sub_menus:
                        menu_string += new_menu.get_menus(new_menu.menus)
        return menu_string



# conversion.py

import os

from src.models._base_def import AField


def creates_conversions(conversions, obj_dir) -> None:
    path = "{}/conversions".format(obj_dir)
    if not os.path.exists(path.encode("utf-8")):
        os.makedirs(path.encode("utf-8"))
    for conversion in conversions:
        ImportMap(conversion, path).to_file()


class ImportMap(AField):
    """ " """

    def __init__(self, data, path) -> None:
        super().__init__(data, path)
        self.dest_def_id = data["destDefId"]
        self.check = data["checkRequiredFields"]

    def to_file(self) -> None:
        super().to_file()

        file = open("{}/PROPERTIES.md".format(self.dir), "w+", encoding="utf-8")
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- destDefId : {}\n".format(self.dest_def_id))
            file.write("- checkRequiredFields : {}\n".format(self.check))

            if self.columns:
                # dest="process" col="-10" uniqueId="-1"
                file.write("\n\n## Columns\n")
                file.write("|dest|col|uniqueId|\n")
                file.write("|---|---|---|\n")
                for c in self.columns:
                    file.write(
                        ("|{}|{}|{}|\n").format(c["dest"], c["col"], c["uniqueId"])
                    )



import re


def split_name(string, max=None):
    s = re.sub("((?![\w]).)", "_", string)  # supression des charactères spéciaux
    s = re.sub("([_-]+)", "_", s)  # suppression des répétitions de undersocre et tiret
    s = s.rstrip("_").rstrip("-")
    if max:  # réduire la taille de la chaine de caractère si on défini un max
        s = s[:max] + "~" if len(s) > max else s
    return s



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages, Extension

with open('README.md', 'r') as f:
    readme = f.read()

with open('LICENSE', 'r') as f:
    licence = f.read()

import re
VERSIONFILE="src/__version__.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    verstr = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

setup(
    name='xmlib-to-git',
    version=verstr,
    description='Generates a Git repository from a Infinite Blue Application XML file',
    long_description_content_type='text/markdown',
    long_description=readme,
    author='David Dessertine',
    license=licence,
    author_email='david.dessertine@foederis.fr',
    packages=find_packages(exclude=('tests', 'docs')),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independant'
    ],
    install_requires=['untangle', 'lxml', 'gitpython', 'unidecode', 'Pillow'],
    entry_points={
        'console_scripts': ['xmlib-to-git=src.__main__:main'],
    },
    include_package_data=True
)




# -*- coding: utf-8 -*-

import os, re
from .section import Section
from src.utils import split_tool

    
class WebPage(object):
    """
    Défini une page Rollase (webpage)
    """
    sections = ""

    def __init__(self, webpage, webpagedef_dir):
        self.id = webpage["id"]
        self.orig_id = webpage["origId"]
        self.page_def_id = webpage["pageDefId"]
        self.is_base_version = webpage["isBasedVersion"]
        self.is_login_only = webpage["isLoginOnly"]
        self.name = webpage.PageName.cdata
        if hasattr(webpage, "PageSections"):
            self.sections = webpage.PageSections.PageSection
        dirname = split_tool.split_name(self.name, 50)
        self.page_dir = "{}/{}".format(webpagedef_dir, dirname)

    def to_file(self):
        if not os.path.exists(self.page_dir.encode('utf-8')):
            os.makedirs(self.page_dir.encode('utf-8'))
        # fichier de propriétés
        if not os.path.exists(self.page_dir.encode('utf-8')):
            os.makedirs(self.page_dir.encode('utf-8'))
        file = open("{}/PROPERTIES.md".format(self.page_dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- pageDefId : {}\n".format(self.page_def_id))
            file.write("- isBasedVersion : {}\n".format(self.is_base_version))
            file.write("- isLoginOnly : {}\n".format(self.is_login_only))

        if self.sections:
            for section in self.sections:
                new_section = Section(section, self.page_dir)
                new_section.to_file()



# batch_job.py


from src import settings
from .object_def.event import Event


def creates_batchs(batchs) -> None:
    # jobs
    for batch in batchs:
        event = Event(batch, settings.APP_DIR + "/jobs/")
        event.to_file()



# -*- coding: utf-8 -*-
from .cell import Cell
import os, re
from src.utils import split_tool


class Section(object):
    """
    Une section d'un page web
    """

    section_title = ""
    translations = ""
    cells = ""

    def __init__(self, section, page_directory):
        self.id = section["id"]
        self.orig_id = section["origId"]
        self.page_id = section["pageId"]
        self.order_no = section["orderNo"]
        self.border_style = section["borderStyle"]
        self.columns = section["columns"]
        self.view_tab_no = section["viewTabNo"]
        self.page_col_no = section["pageColNo"]
        self.is_collapsable = section["isCollapsable"]
        self.is_default = section["isDefault"]
        self.hide_mobile = section["hideMobile"]
        self.do_not_synch = section["doNotSynch"]
        if hasattr(section, "SectionTitle"):
            self.section_title = section.SectionTitle.cdata
        if hasattr(section, "Translations"):
            if hasattr(section.Translations, "Translation"):
                self.translations = section.Translations.Translation
        if hasattr(section, "PageCells"):
            if hasattr(section.PageCells, "PageCell"):
                self.cells = section.PageCells.PageCell
        dirname = split_tool.split_name(self.section_title, 50)
        self.section_directory = "{}/{}".format(page_directory, dirname)

    def to_file(self):
        # fichier de propriétés
        if not os.path.exists(self.section_directory.encode('utf-8')):
            os.makedirs(self.section_directory.encode('utf-8'))
        with open("{}/PROPERTIES.md".format(self.section_directory), 'w+', encoding='utf-8') as file:
            # nom de l'objet
            file.write("# {}\n".format(self.section_title))

            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- pageId : {}\n".format(self.page_id))
            file.write("- orderNo : {}\n".format(self.order_no))
            file.write("- borderStyle : {}\n".format(self.border_style))
            file.write("- columns : {}\n".format(self.columns))
            file.write("- viewTabNo : {}\n".format(self.view_tab_no))
            file.write("- pageColNo : {}\n".format(self.page_col_no))
            file.write("- [{}] isCollapsable\n".format("X" if self.is_collapsable == "true" else " "))
            file.write("- [{}] isDefault\n".format("X" if self.is_default == "true" else " "))
            file.write("- [{}] hideMobile\n".format("X" if self.hide_mobile == "true" else " "))
            file.write("- [{}] doNotSynch\n".format("X" if self.do_not_synch == "true" else " "))

            if self.translations:
                file.write("## Traductions\n")
                file.write("|code|nom|text|\n")
                file.write("|---|---|---|\n")
                for trans in self.translations:
                    file.write("|{}|{}|{}|\n".format(trans["langCode"], trans["fieldName"], trans["text"]))

        if self.cells:
            for cell in self.cells:
                new_cell = Cell(cell, self.section_directory)
                new_cell.to_file()



# logo.py

from src import settings
import base64

def creates_logo(encoded_str, name):
    ext = name.split('.')[1]
    with open("{}/logo.{}".format(settings.APP_DIR, ext), "wb") as file:
        file_content = base64.b64decode(encoded_str)
        file.write(file_content)


# token.py

from src import settings

def creates_tokens(token_names):
    t = open("{}/TOKENS.md".format(settings.APP_DIR), "w+", encoding="utf-8")
    with t:
        t.write("\n## Champs de fusion :\n")
        for x in token_names.split("|"):
            if len(x) > 0:
                t.write("- {}\n".format(x))


# workflow.py


def creates_workflow(worflows, dir) -> None:
    t = open("{}/NODE.md".format(dir), "w+", encoding="utf-8")
    with t:
        t.write("\n## Node in workflow :\n")
        t.write("|s|a|\n|---|---|\n")
        for x in worflows:
            t.write("|{}|{}|\n".format(x["s"], x["a"]))



# button.py

import os

from src.models._base_def import AField

def creates_buttons(buttons, obj_dir) -> None:
    path = "{}/buttons".format(obj_dir)
    if not os.path.exists(path.encode('utf-8')):
        os.makedirs(path.encode('utf-8'))
    for button in buttons:
        Button(button, path).to_file()

class Button(AField):
    """ 
    Creates buttons
    """
    
    def __init__(self, data, path) -> None:
        super().__init__(data, path)
        self.integration_name = data["name"]
        
    def to_file(self) -> None:
        super().to_file()
    
        file = open("{}/PROPERTIES.md".format(self.dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDefId : {}\n".format(self.obj_def_id))
            file.write("- name : {}\n".format(self.integration_name))
            
                


# view.py

import os
from src.models._base_def import AField
from src.utils import split_tool

def creates_views(views, obj_dir) -> None:
        path = "{}/views".format(obj_dir)
        if not os.path.exists(path.encode('utf-8')):
            os.makedirs(path.encode('utf-8'))
        
        from .view import View
        for view in views:
            new_view = View(view, path)
            new_view.to_file()

class View(AField):
    """

    """

    properties = ""

    def __init__(self, data, path):
        super().__init__(data, path)

        self.order = data["orderNo"]
        self.is_system = data["isSystem"]
        self.columns = data.Columns.cdata.replace("\n","")

    def to_file(self):
        super().to_file()
        
        file = open("{}/PROPERTIES.md".format(self.dir), 'w+', encoding='utf-8')
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- orderNo : {}\n".format(self.orig_id))
            file.write("- isSystem : {}\n".format(self.is_system))

            # Propriétés
            if self.properties:
                file.write("\n\n## Properties\n")
                for el in self.properties["children"]:
                    file.write("- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"]))

            #Colonnes
            file.write("\n\n## Columns\n")
            for col in self.columns.split(","):
                if col and col != " ":
                    file.write("- {}\n".format(col))





# event.py
import os
from src.models._base_def import AField


def creates_events(events, obj_dir) -> None:
    # créer un dossier events + nouveaux events
    path = "{}/events".format(obj_dir)
    if not os.path.exists(path.encode("utf-8")):
        os.makedirs(path.encode("utf-8"))
    from .event import Event

    for event in events:
        new_event = Event(event, path)
        new_event.to_file()


class Event(AField):
    """
    Define an event
    """

    script = ""

    def __init__(self, data, path) -> None:
        super().__init__(data, path)
        self.order = data["orderNo"]
        self.config_id = data["configId"]
        self.name = data["name"]
        self.on_what = data["onWhat"]  # sur quel origId d'un objet
        self.relative_to_id = data["relativeToId"]
        self.delay = data["delay"]
        self.template_id = data["templateId"]
        self.change_field_id = data["changeFieldId"]
        self.id_deployed = data["isDeployed"]

    def to_file(self) -> None:
        super().to_file()
        # properties file
        if not os.path.exists(self.dir.encode("utf-8")):
            os.makedirs(self.dir.encode("utf-8"))
        file = open("{}/PROPERTIES.md".format(self.dir), "w+", encoding="utf-8")
        with file:
            # nom de l'objet
            file.write("# {}\n".format(self.name))
            # Informations
            file.write("\n\n## Informations\n")
            file.write("- id : {}\n".format(self.id))
            file.write("- origId : {}\n".format(self.orig_id))
            file.write("- objDef : {}\n".format(self.obj_def_id))
            file.write("- configId : {}\n".format(self.id))
            file.write(
                "- onWhat : {}\n".format(self.orig_id)
            )  # TODO : trouver le nom dans un dict
            file.write("- relativeTo : {}\n".format(self.relative_to_id))  # TODO: idem
            file.write("- delay : {}\n".format(self.delay))
            file.write("- templateId : {}\n".format(self.template_id))  # TODO: idem
            file.write(
                "- changeField : {}\n".format(self.change_field_id)
            )  # TODO: idem

            # Properties
            file.write("\n\n## Properties\n")
            for el in self.properties["children"]:
                if (el.__dict__["_name"] != "ConditionB64") and (
                    el.__dict__["_name"] != "Props"
                ):
                    file.write(
                        "- {} : {}\n".format(el.__dict__["_name"], el.__dict__["cdata"])
                    )

            if self.properties:
                for el in self.properties["children"]:
                    if el.__dict__["_name"] == "template":
                        if el.__dict__["cdata"] != self.script:
                            self.template(el.__dict__["cdata"])
                    else:
                        file.write(
                            "- {} : {}\n".format(
                                el.__dict__["_name"], el.__dict__["cdata"]
                            )
                        )
